package NTLSystem;

import java.util.*;

/**
 * An interface for objects which wish to contain
 * EditableDataElements and use the EditableDataManipulator.  This
 * interface defines the methods the EditableDataManipulator needs to
 * manipulate data in the Container.
 * <P>
 * $Id: EditableDataContainer.java,v 1.1 1998/07/02 02:31:12 jsled Exp $<BR>
 * Last editor: $Author: jsled $ 
 *
 * @author  Joshua Sled [jsled@icsi.berkeley.edu]
 * @see     EditableDataElement
 * @see     NTLSystem.support.EditableDataManipulator
 * @version $Revision: 1.1 $
 **/
public interface EditableDataContainer
{
  /**
   * Requests the addition of the given named element from the
   * EditableDataDictionary to the EditableDataContainer.  It is the
   * responsibility of the EditableDataContainer to get the requested 
   * object, create it's instance of it and add it to the container
   * appropriately. 
   *
   * @param     name The name of the EditableDataElement from the
   *            EditableDataDictionary to add to the
   *            EditableDataContainer. 
   **/
  public void addElement(String name);
  
  /**
   * Requests that the named element be removed from the
   * EditableDataContainer.  It is the responsbility of the
   * EditableDataContainer to remove the object as requested.
   *
   * @param     name The name of the EditableDataElement to remove
   *            from the EditableDataContainer.
   **/
  public void removeElement(String name);
  
  /**
   * @return    An Enumeration of Strings consisting of the names of
   *            all the EditableDataElements from the dictionary which
   *            are present in the EditableDataContainer.
   **/
  public Enumeration getExistingElements();
}
