package NTLSystem;

//import com.sun.java.swing.*;
//import java.awt.*;
//import java.awt.event.*;
import java.io.*;
//import java.lang.reflect.*;
import java.util.*;
import NTLSystem.support.*;

/**
 * A lock object for use in locking any object, particularly Fstructs.
 *
 * <P>
 * $Id: Lock.java,v 1.4 1998/11/24 00:20:49 tajh Exp $<BR>
 * Last editor: $Author: tajh $
 *
 * @author     Tajh Taylor [tajh@icsi.berkeley.edu]
 * @see        LockManager
 * @see        feature.Fstruct
 * @version    $Revision: 1.4 $
 **/
public class Lock
{
   //==========================
   // ATTRIBUTES

   /** The ID number for the lock, which is automatically generated */
   private long id;

   /** A character indicating the type of lock. */
   private char type;

   /** A way of maintaining lock uniqueness (to 64 bits). */
   private static long lastId;

   /** The object that this lock is a lock for. */
   private Object item;

   //==========================
   // CONSTRUCTORS

   // A static constructor to set up ID maintenance.
   static
   {
      lastId = 0L;
   }
  
   /** A simple constructor for a lock which is typeless.
       @param newItem The object to be locked. */
   public Lock(Object newItem)
   {
      id = getNextId();
      item = newItem;
      type = '\0';
   }

   /** The more useful constructor for a lock that allows lock type
          specification.
       @param newItem The object to be locked.
       @param newType  A character indicating the type of the lock. */
   public Lock(Object newItem, char newType)
   {
      this(newItem);
      type = Character.toLowerCase(newType);
   }

   //==========================
   // UTILITY METHODS 

   /** Get a unique ID number for a lock.
       @return A uniquely generated ID number for a lock. */
   private static synchronized long getNextId()
   {
      lastId++;
      return lastId;
   }

   /** Get the lock's ID number.
       @return A long containing the lock ID. */
   public long getId()
   {
      return id;
   }

   /** Get the lock's ID as a Long object.
       Intended for use as a hashtable key.
       @return A Long object containing the lock ID. */
   public Long getIdObject()
   {
      return new Long(id);
   }
  
   /** Get the lock's type.
       @return A char indicating the lock type. */
   public char getType()
   {
      return type;
   }

   /** Get the item that this lock is a lock for.
       The return type will require casting.
       @return The object that this is a lock on. */
   public Object getItem()
   {
      return item;
   }
}


