package NTLSystem.support;

import java.io.*;
import java.util.*;
import java.net.*;
import java.awt.image.*;
import java.awt.*;
import NTLSystem.*;

/**
 * Interface to file contents which goes through either the file system
 * or through the web depending on how it's set up.
 * <P>
 *  // NOTE: probably temporary!! 
 * $Id: DataSource.java,v 1.3 1998/07/23 06:25:44 nchang Exp $<BR>
 * Last editor: $Author: nchang $
 *
 * @author    David Bailey [dbailey@icsi.berkeley.edu]
 * @version   $Revision: 1.3 $
 **/

public class DataSource {
  
  private static boolean useWeb;
  private static String base;

  private static final String NOT_FOUND_MESSAGE = "404 Not Found";

  public static void setUseWeb(String s) { 
    useWeb = true;
    base = s;
  }

  public static void setUseFiles(String s) { 
    useWeb = false;
    base = s;
  }

  public static String getBase() { return base; }

  // Returns array of entries in directory 's', excluding entries
  // ending with a '~'.
  public static String[] getDirectoryList(String s) 
    throws MalformedURLException, IOException 
   {
   if (useWeb) { 
 	// This is gross but should work...
 	Vector entries = new Vector();  // Vector<String>
 	StringBuffer text = new StringBuffer();
 	InputStream is = new URL(base + s).openStream();
 	int ch = is.read();
 	while (ch != -1) {
 	  text.append((char)ch);
 	  ch = is.read();
 	}
 	StringTokenizer tokEnum = new StringTokenizer(text.toString());
 	while (tokEnum.hasMoreTokens()) {
 	  String tok = tokEnum.nextToken();
 	  if (tok.startsWith("NAME=\"") && !tok.endsWith("~\"")) {
 	    String e = tok.substring(6, tok.length() - 1);
 	    if (e.charAt(e.length() - 1) == '/') {
 	      e = e.substring(0, e.length() - 1);
 	    }
 	    entries.addElement(e);
 	  }
 	} // end while over tokEnum
 	String[] res = new String[entries.size()];
 	for (int i = 0; i < entries.size(); i++) {
 	  res[i] = (String)entries.elementAt(i);
 	}
 	return res;
    } // end if (useWeb)
    else {
	File f = new File(base + s);
	if (f.isDirectory()) {
	  String[] fullList = f.list();
	  Vector v = new Vector();
	  for (int i = 0; i < fullList.length; i++) {
	    if (!fullList[i].endsWith("~")) {
	      v.addElement(fullList[i]);
	    }
	  }
	  String[] res = new String[v.size()];
	  for (int i = 0; i < v.size(); i++) {
	    res[i] = (String)v.elementAt(i);
	  }
	  return res;
	}
	else {
	  // SHOULD THROW EXCEPTION?
	  return null;
	}
      }
    }
  
  public static InputStream getInputStream(String s) 
    throws MalformedURLException, IOException, FileNotFoundException
      {
     if (useWeb) {
       return new URL(base + s).openStream();
     }
     else {
      return new FileInputStream(base + s);
    }
  }


  public static NestedStreamTokenizer getNestedStreamTokenizer(String s) 
       throws MalformedURLException, IOException, FileNotFoundException
  {
    if (useWeb) {
      return new NestedStreamTokenizer(new StringReader(((String)new
			       URL(base + s).getContent())));
    }
    else {
      return new NestedStreamTokenizer(new FileReader(base + s));
    }
  }

  // The Component ought to be the containing one, I suppose - but not sure.
  public static Image getLoadedImage(String s, Component c) 
              throws MalformedURLException, InterruptedException 
  {
    Image i;
    if (useWeb) {
            i = Toolkit.getDefaultToolkit().getImage(new URL(base + s));
    }
    else {
      i = Toolkit.getDefaultToolkit().getImage(base + s);
    }
    MediaTracker t = new MediaTracker(c);
    t.addImage(i, 0);
    t.waitForAll();
    return i;
  }

  public static boolean exists(String s) {
    if (useWeb) {
      // How else, but to try to load it and then see if the stream
      // contains "404 Not Found" ???
      boolean exists;
      try {
	InputStream is = new URL(base + s).openStream();
	StringBuffer text = new StringBuffer();
	int ch = is.read();
	while (ch != -1) {
	  text.append((char)ch);
	  ch = is.read();
	}
	exists = (text.toString().indexOf(NOT_FOUND_MESSAGE) < 0);
      }
      catch (Exception e) {
	exists = false;
      }
      return exists;
    }
    else {
      return new File(base + s).exists();
    }
  }

}
