package NTLSystem.support;

import com.sun.java.swing.*;
import java.beans.*;
import java.awt.*;
import java.awt.event.*;

/**
 * A class which will edit a collection of properties given as
 * name/value pairs.  This code will use the JavaBeans editors for the
 * various simple data types [Int, Float, etc], and will use the
 * PropertyEditorManager to get the editors for other types, thus
 * allowing the easy extension of the PropertyEditor.
 * <P>
 * Most of this code was stolen from the BDK BeanBox's
 * PropertySheet.java file.  However, it has been heavily modifed to
 * account for changes in implementation.
 * <P>
 * $Id: PropertyCollectionSheet.java,v 1.10 1998/07/31 21:07:03 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author Josh Sled [jsled@icsi.berkeley.edu]
 * @see java.beans.PropertyEditor
 * @see java.beans.PropertyEditorManager
 * @version $Revision: 1.10 $
 *
 **/
public class PropertyCollectionSheet
extends JFrame 
implements ActionListener
{

  /* ATTRIBUTES */

  /**
   * The panel which actually contains the property-value pairs for
   * editing.
   **/
  private PropertyCollectionSheetPanel panel;
  /** A flag to denote if editing has started. **/
  private boolean started;
  
  /**
   * A constructor for the PropertyCollectionSheet.  Uses the
   * PropertyCollection interface to get the relevant details from an
   * editable target. 
   * 
   * @param     target The PropertyCollection implementer to be
   *            edited.
   * @param     x The initial x coord. of the editing window.
   * @param     y The initial y coord. of the editing window.
   **/
  public PropertyCollectionSheet(PropertyCollection target, int x, int y) {
    super("Properties - <initializing...>");

    GridBagLayout gbl = new GridBagLayout();
    GridBagConstraints gbc = new GridBagConstraints();
    getContentPane().setLayout(gbl);

    setTitle("Properties - " + target.getLabel());
    setBounds(x, y, 500, 700);

    gbc.gridwidth = GridBagConstraints.REMAINDER;
    gbc.gridheight = 1;

    panel = new PropertyCollectionSheetPanel( target );
    JScrollPane jsp = new JScrollPane(panel);
    gbl.setConstraints(jsp, gbc);
    getContentPane().add(jsp);

    // Add Ok button
    JPanel p = new JPanel();
    JButton b;

    b = new JButton("Ok");
    b.addActionListener(this);
    p.add(b);

    b = new JButton("Apply");
    b.addActionListener(this);
    p.add(b);

    b = new JButton("Cancel");
    b.addActionListener(this);
    p.add(b);

    p.setVisible(true);
    gbl.setConstraints(p, gbc);
    getContentPane().add(p);
    addWindowListener(new WindowCloser(this, false));

    invalidate();
    pack();

    setVisible(true); 
    started = true;

  }

  /**
   * Sets the editing target.  This mostly calls the panel's setTarget
   * method.
   *
   * @param     targ The PropertyCollection implementer being edited.
   **/
  public void setTarget(PropertyCollection targ) {
    String displayName = targ.getLabel();
    panel.setTarget(targ);
    setTitle("Properties - " + displayName);
  }

  /**
   * Saves the values in the PropertyCollectionSheet back to their
   * PropertyCollection.  Until this is called, the values in the
   * PropertyCollection are untouched, and only the getter method has
   * been called on the PropertyCollection.
   *
   * @return    True if the values were able to be saved, false
   *            otherwise.
   **/
  public boolean saveValues() {
    return panel.saveValues();
  }

  void wasModified(PropertyChangeEvent evt) {
    panel.wasModified(evt);
  }

  /**
   * Required for implementation of ActionListener interface.
   **/ 
  public void actionPerformed(ActionEvent e)
  {
    if (e.getActionCommand().equals("Ok")) {
      // Save the data and close the window.
      if (this.saveValues())
	this.dispatchEvent(new WindowEvent(this,
					   WindowEvent.WINDOW_CLOSING));
    } else if (e.getActionCommand().equals("Apply")) {
      // Just save the data.
      this.saveValues();
    } else if (e.getActionCommand().equals("Cancel")) {
      // Just close the window.
      this.dispatchEvent(new WindowEvent(this, WindowEvent.WINDOW_CLOSING));
    }
  }

}

