package NTLSystem.support;

import com.sun.java.swing.*;
import com.sun.java.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.text.*;
import java.util.*;

/**
 * Generic, highly-reuseable utility functions.
 * <P>
 * $Id: Utility.java,v 1.4 1998/09/03 02:47:17 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author Josh Sled [jsled@icsi.berkeley.edu]
 * @version $Revision: 1.4 $
 **/
public class Utility
{
  /**
   * A utility function to simply create MenuItem and add it to the
   * given menu. 
   *
   * @param     m The menu to add the item to.
   * @param     itemName The name of the menu item
   * @param     actionCmd The action command for the menu item.
   *            Null if it should be the same as the itemName [default
   *            behavior].
   * @param     al The object which will listen for this
   *            menu item's selection.
   * @param     enabled True if the item is enable, false if it is
   *            greyed out.
   **/
  public static void AddMenuItem(Menu m, String itemName,
				 String actionCmd, 
				 ActionListener al,
				 boolean enabled)
  {
    MenuItem mi = new MenuItem(itemName);
    if (actionCmd != null) {
      mi.setActionCommand(actionCmd);
    }
    mi.setEnabled(enabled);
    mi.addActionListener(al);
    m.add(mi);
  }

  /** 
   * Convenience function for above.  Creates enabled menu items.
   * 
   * @param     m The menu to add the item to.
   * @param     itemName The name of the MenuItem.
   * @param     actionCmd The action command for this MenuItem.
   * @param     al The ActionListener which will listen for this
   *            MenuItem's selection.
   **/
  public static void AddMenuItem(Menu m, String itemName,
				 String actionCmd,
				 ActionListener al)
  {
    AddMenuItem(m, itemName, actionCmd, al, true);
  }

    /**
     * Saves the output of a JTextArea to the given Writer.  All
     * exceptions are tossed back up.
     *
     * @param     jta The JTextArea to save the contents of.
     * @param     w The Writer to save the contents to.
     * @exception IOException If the underlying Writer throws one on writing.
     **/
    public static void SaveTextArea(JTextArea jta,
				    Writer w)
	throws IOException
    {
	try {
	    int BLOCK_SIZE = 16384;
	    Document d = jta.getDocument();
	    String s;
	    int i=0;
	    int docLength = d.getLength();
	    // Save it in 16k chunks.
	    for (; (i+BLOCK_SIZE)<docLength; i += BLOCK_SIZE) {
		s = d.getText( i, BLOCK_SIZE );
		w.write( s );
	    } // end for over i
	    // Get the last truncated chunk.
	    s = d.getText( i, docLength-i );
	    w.write( s );
	    w.flush();
	} catch (BadLocationException ble) {
	    // We don't expect this with the
	    // brain-dead arguments given.
	    throw new InternalError( ble.getMessage() );
	}
    }

}

