package NTLSystem;

import com.sun.java.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import NTLSystem.support.*;

///////////////////////////////////////////////////////////////////////
// Holds the current "setup" of the application.

class Context extends Observable implements Serializable {

  private String[] langslotList;  // each of the form "English2"
  private String[] paramList;
  private String[] datasetList;
  private String[] lexiconList; // for the current language

  private ScenarioSet scenarios;
  private String lang;     // e.g., "English"
  private int numSlots;    // e.g., 2
  private Params params;
  private Dataset dataset;
  private Lexicon lexicon;

  // Call this immediately after creation.
  public void init() {
    // Set up scenarios:
    setParams(VerbLearn.INITIAL_PARAM_FILE);  // probably "default"
    scenarios = new ScenarioSet();
    rereadLists();
  }

  public void rereadLists() {
    try {
      langslotList = DataSource.getDirectoryList("lang");
      paramList = DataSource.getDirectoryList("param");
      datasetList = DataSource.getDirectoryList("dataset");
      if (lang != null) {
	lexiconList = DataSource.getDirectoryList("lang/" + langslot() +
						  "/lexicon");
      }
    }
    catch (Exception e) {
      System.err.println("Context couldn't read directories:" +
			 e.getMessage());
    }
    setChanged(); notifyObservers(); 
    if (VerbLearn.frame() != null) {
      VerbLearn.frame().validate();
    }
  }    

  public String[] langslotList() { return langslotList; }

  public String[] paramList() { return paramList; }

  public String[] datasetList() { return datasetList; }

  public String[] lexiconList() { return lexiconList; }

  public ScenarioSet scenarios() { return scenarios; }

  public String langslot() { 
    if (lang != null) return lang + numSlots; 
    else return null;
  }

  public String lang() { return lang; }

  public int numSlots() { return numSlots; }

  public Dataset dataset() { return dataset; }

  public Params params() { return params; }

  public Lexicon lexicon() { return lexicon; }

  // If argument is of the form "English2", set lang and numSlots
  // accordingly.  If argument is null, unset lang and numSlots.  In
  // either case, unset dataset and lexicon.
  public void setLang(String langslot) {
    VerbLearn.labelPanel().saveLabels();    // use GUI to confirm
    if (langslot == null) {
      lang = null;
      numSlots = 0;
      lexiconList = null;
      scenarios.clearLabelList(); // nc added
      //      System.out.println("cleared label lists -- langslot null");
    }
    else {
      scenarios.clearLabelList(); // nc added
      //      System.out.println("cleared label lists, making new -- not null");

      int i;
      for (i = 0; !Character.isDigit(langslot.charAt(i)); i++) {}
      lang = langslot.substring(0, i);
      numSlots = Integer.parseInt(langslot.substring(i));

      scenarios.newLabelList();
      scenarios.loadLabels();
      try {
	lexiconList = DataSource.getDirectoryList("lang/" + langslot +
						  "/lexicon");
      }
      catch (Exception e) {
	System.err.println("Context couldn't read language directory:" +
			   e.getMessage());
      }
    }
    privateSetLexicon(null);
    setChanged(); notifyObservers(); VerbLearn.frame().validate();
  }

  // Set params from the file p, or to null if p is null.
  // Set params to the already existing Param object 'p'.
  public void setParams(Params p) {
    params = p;
    // nc added to ensure that scenarios have correct on/off nature
    VerbLearn.context().scenarios().rearrangeFstructs();    
    setChanged(); notifyObservers(); VerbLearn.frame().validate();
  }

  public void setParams(String p) {

    // EVENTUALLY, WILL NEED TO OPTIONALLY SAVE CURRENT PARAMS
    // nc modified to query first

    if (params != null) {
      if (VerbLearn.context().params().isModified()) {
	// pop up little dialog box to ask whether to save current or not

	YesNoDialog confirm = new YesNoDialog(VerbLearn.frame(), "Save changes", 
	      "Save changes made to " + VerbLearn.context().params().id() +
		      " before loading new parameters file?",
					      "Yes", "No", null);
	confirm.show();
	if (confirm.answer() == confirm.CANCEL) return;
	if (confirm.answer() == confirm.YES) {
	  VerbLearn.context().params().save();
	}
      }
    }
    if (p == null) {
      params = null;
    }
    else {
      try {
	params = new Params(DataSource.
			    getNestedStreamTokenizer("param/" + p), p);
      }
      catch (IOException e) {}
    }
    if (VerbLearn.context().scenarios() != null) {
      VerbLearn.context().scenarios().rearrangeFstructs();    // nc
    }
    setChanged(); notifyObservers(); 
    if (VerbLearn.frame() != null) {
      VerbLearn.frame().validate();
    }
  }
  
  // nc added to update current parameters based on hashtable
  //created during edit params
  // don't need this: only if feat set changed:
  //    VerbLearn.context().scenarios().rearrangeFstructs();    
  public void setParams(Hashtable pTable) {
    ValuePanel vp;
    boolean changed;
    Object newValue;

    Enumeration pEnum = pTable.keys();
    while (pEnum.hasMoreElements()) {
      String p = (String)pEnum.nextElement();
      vp = (ValuePanel)pTable.get(p);

      changed = vp.changed();
      if (changed) {
	newValue = new Object();
	newValue = (Object)vp.value();
	//	System.out.println("Feature " + p + " changed from " +
	//   VerbLearn.context().params().paramsValues().get(p).toString() +
	//   " to..." + newValue.toString());
	VerbLearn.context().params().paramsValues().remove(p);
	VerbLearn.context().params().paramsValues().put(p, newValue);
	VerbLearn.context().params().setModified(true);
      }
    }
    setChanged(); notifyObservers(); 
    if (VerbLearn.frame() != null) {
      VerbLearn.frame().validate();
    }
  }

  // Set params to the already existing Dataset object 'd'. // nc datset?
  public void setDataset(Dataset d) {
    dataset = d;
    setChanged(); notifyObservers(); VerbLearn.frame().validate();
  }

  // Set dataset from the file d, or to null if d is null.
  public void setDataset(String d) {
    privateSetDataset(d);
    setChanged(); notifyObservers(); VerbLearn.frame().validate();
  }

  public void updateScenarioSet(){
     setChanged(); notifyObservers(); VerbLearn.frame().validate();
  }

  private void privateSetDataset(String d) {
    // EVENTUALLY, WILL NEED TO OPTIONALLY SAVE CURRENT DATASET
    if (d == null) {
      dataset = null;
    }
    else {
      try {
	dataset = new Dataset(DataSource.
			      getNestedStreamTokenizer("dataset/" + d), d);
      }
      catch (IOException e) {}
    }
  }

  // Set lexicon from the file l, or to an empty lexicon if l is "New/Unnamed",
  // or to null if l is null.
  public void setLexicon(String l) {
    privateSetLexicon(l);
    setChanged(); notifyObservers(); VerbLearn.frame().validate();
  }

  private void privateSetLexicon(String l) {
    if (l == null) {
      lexicon = null;
    }
    else if (l.equals("New/Unnamed")) {
      lexicon = new Lexicon();
    }
    else {
      try {
	ObjectInputStream is = 
	  new ObjectInputStream(DataSource.
				getInputStream("lang/" + langslot() + 
					       "/lexicon/" + l));
	lexicon = (Lexicon)is.readObject();

      }
      catch (Exception e) {
	System.err.println("Exception loading lexicon: " + e.getMessage());
      }
    }
  }
}
