package NTLSystem.verblearn

import java.io.*;
import java.util.*;
import NTLSystem.support.*;


///////////////////////////////////////////////////////////////////////
// Probabilistic Linking Feature Structures, used for word senses.
// IN THIS VERSION, WE ASSUME ALL PFSTRUCTS ARE OVER ALL LINKING FEATURES.

class PFstruct implements Serializable {

  protected Hashtable dists;  
  // Should be "private protected" but it's gone w/Java 1.1...

  // Construct from a Fstruct and a set of virtual sample counts.
  PFstruct(Fstruct f, Hashtable v) {
    dists = new Hashtable();
    // First, add the "virtual samples":
    Enumeration fEnum = v.keys();
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      dists.put(fname, new Dist(fname, ((Double)v.get(fname)).doubleValue()));
    }
    // Next, add the actual sample, f, by incrementing the appropriate values:
    fEnum = f.features();
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      ((Dist)dists.get(fname)).incrementValue((String)f.get(fname));
    }
  }

  // Construct by merging two PFstructs.
  PFstruct(PFstruct pf1, PFstruct pf2, Hashtable v) {
    dists = new Hashtable();
    Enumeration fEnum = pf1.dists.keys();
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      dists.put(fname, new Dist(fname, 
				(Dist)pf1.dists.get(fname), 
				(Dist)pf2.dists.get(fname), 
				((Double)v.get(fname)).doubleValue()));
    }
  }

  public double likelihood(Fstruct f) {
    double res = 1;
    Enumeration fEnum = f.features();
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      res = res * ((Dist)dists.get(fname)).likelihood(f.get(fname));
    }
    return res;
  }

  // A heuristic measure of the similarity between two probabilistic
  // fstructs.  Currently we multiply the similarities between the
  // distributions for each feature but this may or may not be the
  // best combination function.
  public double similarityTo(PFstruct pf) {
    double res = 1;
    Enumeration fEnum = dists.keys();
    while (fEnum.hasMoreElements()) {
      String fname = (String)fEnum.nextElement();
      res = res * ((Dist)dists.get(fname)).
	similarityTo((Dist)pf.dists.get(fname));
    }
    return res;
  }

  public Dist getDist(String f) { return (Dist)dists.get(f); }

}
