package NTLSystem.verblearn;

import com.sun.java.swing.*;
import com.sun.java.swing.event.*;
import com.sun.java.swing.text.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import NTLSystem.*;
import NTLSystem.features.*;
import NTLSystem.support.*;


/**
 * The GUI functionality for an obey test.
 *
 * $Id: ObeyGUI.java,v 1.2 1998/10/16 01:59:37 jsled Exp $<BR>
 * Last editor: $Author: jsled $
 *
 * @author    Joshua Sled [jsled@icsi.berkeley.edu]
 * @see       Lexicon
 * @see       ScenarioSet
 * @see       VerbLearn
 * @version   $Revision: 1.2 $
 **/
public class ObeyGUI
    extends JInternalFrame
{
    
    /** The instance of VerbLearn we are associated with. **/
    private VerbLearn vl;
    
    /** The ScenarioSet of interest. **/
    private ScenarioSet sset;

    /** A combo box for selecting the lexicon of interest. **/
    private JComboBox lexicons;

    /** A ComboBox for selecting the Params to use. **/
    private JComboBox params;

    /** A Checkbox for selecting recognition with or without
	crunchy extra detail. **/
    private JCheckBox detail;

    /** RadioButtons for selecting the testing set. **/
    private JRadioButton allTestSet;
    private JRadioButton selectedTestSet;

    /** A ComboBox for selecting the dataset. **/
    private JComboBox dsets;
    /** A Vector of the datasets in the ScenarioSet. **/
    private Vector dsetNames;
    
    /** A List for selecting a subset of the test set. **/
    private JList testSetList;
    private JScrollPane testSetScroller;
    
    /** Where the obey output will go. **/
    private JTextArea output;

    /** The button which performs obey. **/
    private JButton obeyButton;

    /** The JButton to save the log. **/
    private JButton logSaveButton;
    /** The JButton to clear the log. **/
    private JButton logClearButton;

    /**
     * Constructor.
     *
     * @param     vl The instance of VerbLearn which we are contained
     *            within.
     * @param     initSset The ScenarioSet which is currently being
     *            edited. 
     **/
    public ObeyGUI(VerbLearn vl, ScenarioSet initSset)
    {
	super("Obey", true, true, true, true);

	this.vl = vl;
	this.sset = initSset;
	
	JComponent jc = (JComponent)getContentPane();

	GridBagLayout gbl = new GridBagLayout();
	GridBagConstraints c = new GridBagConstraints();

	jc.setLayout( gbl );

	// The top-left panel with the lexicon selection, lexicon
	// language label and recognition options.

	JPanel jp = new JPanel( new GridBagLayout() );
	
	c.gridwidth = GridBagConstraints.RELATIVE;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	JLabel jl = new JLabel( "Lexicon:" );
	((GridBagLayout)jp.getLayout()).setConstraints( jl, c );
	jp.add( jl );
	
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.HORIZONTAL;
	ComboBoxModel cbm = 
	    new VectorComboBoxModel(sset.getLexicons()) {
		// Adapt the VectorComboBoxModel to return the right
		// thing.
		public Object getElementAt(int i) {
		    return ((Lexicon)super.getElementAt(i)).getName();
		}
	    };
	lexicons = new JComboBox( cbm );
	lexicons.setSelectedIndex( 0 );
	((GridBagLayout)jp.getLayout()).setConstraints( lexicons, c );
	jp.add( lexicons );

	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	c.gridwidth = GridBagConstraints.RELATIVE;
	jl = new JLabel( "Parameters:" );
	((GridBagLayout)jp.getLayout()).setConstraints( jl, c );
	jp.add( jl );
	
	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.REMAINDER;
	cbm = new ComboBoxModelAdaptor( sset.params );
	params = new JComboBox( cbm );
	params.addItemListener( new ItemListener() {
	    public void itemStateChanged(ItemEvent ie) {
		// Set the currentParams key in the ScenarioSet.
		sset.currentParamsKey = (String)params.getSelectedItem();
	    }
	} );
	((GridBagLayout)jp.getLayout()).setConstraints( params, c );
	jp.add( params );


	// Add sub-panel [with cool titled border] for recognize
	// options. 
	JPanel jp2 = new JPanel( new GridBagLayout() );
	jp2.setBorder( BorderFactory.
		       createTitledBorder( "Obey" ) );

	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.HORIZONTAL;
	detail = new JCheckBox( "Detail", false );
	((GridBagLayout)jp2.getLayout()).setConstraints( detail, c );
	jp2.add( detail );

	ButtonGroup bg = new ButtonGroup();

	allTestSet =
	    new JRadioButton( "All test set scenarios", true );
	bg.add( allTestSet );
	((GridBagLayout)jp2.getLayout()).setConstraints( allTestSet,
							 c );
	jp2.add( allTestSet );

	selectedTestSet =
	    new JRadioButton( "Selected test set scenarios", false );
	bg.add( selectedTestSet );
	((GridBagLayout)jp2.getLayout()).
	    setConstraints( selectedTestSet, c );
	jp2.add( selectedTestSet );

	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	((GridBagLayout)jp.getLayout()).setConstraints( jp2, c );
	jp.add( jp2 );
	
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.NONE;
	obeyButton = new JButton( "Obey" );
	obeyButton.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Start recognition on a background thread.
		Thread t = new Thread() {
		    public void run() {
			obeyButton.setEnabled( false );
			obey();
			obeyButton.setEnabled( true );
			JOptionPane.
			showMessageDialog(ObeyGUI.this,
					  "Obeying done.",
					  "Obeying done.",
					  JOptionPane.INFORMATION_MESSAGE);
		    }
		}; // end Thread
		t.start();
	    }
	} );
	((GridBagLayout)jp.getLayout()).setConstraints( obeyButton,
							c );
	jp.add( obeyButton );

	jp2 = new JPanel();
	logSaveButton = new JButton( "Save Log" );
	logSaveButton.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Save the log to a specified file.
		// Get the file name from the user.
		JFileChooser jfc = new JFileChooser();
		jfc.setDialogTitle( "Save training log as..." );
		int status = jfc.showSaveDialog( ObeyGUI.this.vl );
		if ( status != JFileChooser.CANCEL_OPTION ) {
		    File toSaveTo = jfc.getSelectedFile();
		    boolean cont = true;
		    if ( toSaveTo.exists() ) {
			cont = (JOptionPane.showConfirmDialog( ObeyGUI.this.vl,
							       "Overwrite file " +
							       toSaveTo.getPath() +
							       "?",
							       "Overwrite file",
							       JOptionPane.YES_NO_OPTION)
				== JOptionPane.YES_OPTION);
		    } // end if
		    if (cont) {
			try {
			    FileWriter fw =
				new FileWriter( toSaveTo );
			    Utility.SaveTextArea( output, fw );
			    fw.close();
			    JOptionPane.showMessageDialog( ObeyGUI.this.vl,
							   "Done saving log.",
							   "Done saving log.",
							   JOptionPane.INFORMATION_MESSAGE );
			} catch (IOException ioe) {
			    JOptionPane.showMessageDialog( ObeyGUI.this.vl,
							   "Couldn't save file " +
							   toSaveTo.getPath() +
							   " because " +
							   ioe.getMessage(),
							   "File save error",
							   JOptionPane.ERROR_MESSAGE );
			} // end-catch
		    } // end cont
		} // end-if
		// Output the contents of the log to the file.
	    }
	} );
	jp2.add( logSaveButton );

	logClearButton = new JButton( "Clear Log" );
	logClearButton.addActionListener( new ActionListener() {
	    public void actionPerformed(ActionEvent ae) {
		// Clear the log...
		Document d = output.getDocument();
		try {
		    d.remove( 0, d.getLength() );
		} catch (BadLocationException ble) {
		    // We don't expect this because of the brain-dead
		    // arguments we're using, but it might happen.
		    throw new InternalError( ble.getMessage() );
		} // end try-catch
		output.validate();
	    }
	} );
	jp2.add( logClearButton );

	c.fill = GridBagConstraints.HORIZONTAL;
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.anchor = GridBagConstraints.CENTER;
	((GridBagLayout)jp.getLayout()).setConstraints( jp2, c );
	jp.add( jp2 );

	c.gridwidth = GridBagConstraints.RELATIVE;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	c.weightx = c.weighty = 1.0;
	gbl.setConstraints( jp, c );
	jc.add( jp );
	c.weightx = c.weighty = 0.0;


	// Add the top-right panel with the dataset selection and the
	// dataset test-set list.
	jp = new JPanel( new GridBagLayout() );
       
	c.gridwidth = GridBagConstraints.RELATIVE;
	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.WEST;
	jl = new JLabel( "Dataset:" );
	((GridBagLayout)jp.getLayout()).setConstraints( jl, c );
	jp.add( jl );

	Enumeration e = sset.getDatasets();
	dsetNames = new Vector();
	while ( e.hasMoreElements() ) {
	    dsetNames.
		addElement( ((Dataset)e.nextElement()).getName() );
	} // end while over dataset enumeration
	cbm = new VectorComboBoxModel( dsetNames );
	dsets = new JComboBox( cbm );
	dsets.addItemListener( new ItemListener() {
	    public void itemStateChanged(ItemEvent ie) {
		// Change the test set list.
		String dsName = (String)dsets.getSelectedItem();
		Dataset ds = sset.getDataset( dsName );
		if (ds != null) {
		    Enumeration enum = ds.testEnum();
		    Vector v = new Vector();
		    while ( enum.hasMoreElements() ) {
			v.addElement( ((Scenario)enum.nextElement()).getName() );
		    } // end while over test set enumeration
		    testSetList.setListData( v );
		    // Update the GUI list
		    testSetScroller.validate();
		    testSetScroller.update( testSetScroller.getGraphics() );
		} // end if
	    }
	} );
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.HORIZONTAL;
	c.anchor = GridBagConstraints.WEST;
	((GridBagLayout)jp.getLayout()).setConstraints( dsets, c );
	jp.add( dsets );
	
	testSetList = new JList();
	testSetList.setSelectionMode( ListSelectionModel.
				      MULTIPLE_INTERVAL_SELECTION );
	testSetList.setVisibleRowCount( 10 );
	testSetList.setFixedCellWidth( 100 );
	testSetScroller =
	    new JScrollPane( testSetList,
			     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			     JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
	c.gridwidth = GridBagConstraints.REMAINDER;
	c.fill = GridBagConstraints.BOTH;
	c.anchor = GridBagConstraints.CENTER;
	((GridBagLayout)jp.getLayout()).
	    setConstraints( testSetScroller,
			    c );
	jp.add( testSetScroller );
	
	c.weightx = c.weighty = 1.0;
	gbl.setConstraints( jp, c );
	jc.add( jp );

	// Add the text area at the bottom for recognition output.

	output = new AppendScrollingTextArea( 25, 60 );
	output.setEditable( false );
	JScrollPane jsp =
	    new JScrollPane( output,
			     JScrollPane.VERTICAL_SCROLLBAR_ALWAYS,
			     JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
	jsp.setBorder( BorderFactory.createTitledBorder( "Obey log" ) );
	gbl.setConstraints( jsp, c );
	jc.add( jsp );

	// Pad the content pane a bit
	jc.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
	
	// Done... pack'n'show
	pack();
	setVisible( true );

    }

    protected void obey() {
	if (params.getSelectedItem() == null) {
	    output.append("You must choose a parameter file before testing " +
			  "obeying.\n");
	    return;
	}
	if (dsets.getSelectedItem() == null) {
	    output.append("You haven't chosen a dataset!\n");
	    return;
	}

	Object[] scArray = null;
	// Vector<Scenario> to recognize against below.
	Vector toObey = new Vector();
	if ( selectedTestSet.isSelected() ) {
	    // Get the selected scenarios.
	    scArray = testSetList.getSelectedValues();
	    // Copy the elements from the array into the Vector
	    for (int i=0; i<scArray.length; i++) {
		toObey.addElement( sset.
				   getScenario( (String)scArray[i] ) );
	    } // end for over scArray
	} else {
	    // Get all the scenarios from the test set.
	    String dsetName = (String)dsets.getSelectedItem();
	    Dataset d = sset.getDataset( dsetName );
	    if ( d != null ) {
		Enumeration enum = d.testEnum();
		while ( enum.hasMoreElements() ) {
		    toObey.addElement( enum.nextElement() );
		} // end while overe test set numeration 
	    } // end if
	} // end if-else

	if ( toObey.size() == 0 ) {
	    output.append("You haven't chosen any scenarios to test!\n");
	    return;
	}

	output.append("Beginning obey test...\n");
	int numCorrect = 0;
	int numTested = 0;

	Lexicon lex =
	    sset.getLexicon((String)lexicons.getSelectedItem());
	String lang = lex.getLanguage();
	boolean wantDetail = detail.isSelected();
	Fstruct scFstruct, f1, f2;
	VerbComplex scLabelVC;
	String scLabel;
	Scenario sc;
	
	for ( int i=0; i<toObey.size(); i++ ) {
	    sc = (Scenario)toObey.elementAt( i );
	    if (sc == null) {
		output.append( "Scenario is null!?\n" );
	    }
	    if (sc.isLabeled( lang )) {
		scLabelVC = sc.getLabel( lang );
		scLabel = scLabelVC.toString();
		// Derive linking f-struct from the command and world state:
		// f1 = lex.obey(sc.getLabel( lang ),
		// sc.initialLink() ); // FIXME

		// sc.getFstruct(World) is a cheap hack to get the
		// 'initial link' Fstruct.
		Fstruct fs = (Fstruct)sc.getFstruct( "World" ).clone();
		try {
		    // Fix the feature names.
		    Vector v = fs.getAllFeatureNames();
		    Vector vs = fs.getFeatures( v.elements() );
		    Feature f;
		    for (int j=0; j<vs.size(); j++) {
			f = (Feature)vs.elementAt( j );
			fs.removeFeature( f.getName() );
			f.setName( "World." + f.getName() );
		    }
		    fs.putFeatures( vs.elements() );
		} catch (NTLException ntle) {
		    // NoSuchFeatureException won't happen because
		    // we're asking for Features we were just told it
		    // has.  NTLException won't occur because we're
		    // giving it Strings.
		    throw new InternalError( ntle.getMessage() );
		}
		f1 = lex.obey( scLabelVC, fs );

		// Simulate execution by filling in any needed defaults:
		// f2 = execute( f1, sc.initialLink() ); // FIXME

		// Cheap hack to get the 'inital link' Fstruct.
		f2 = execute( f1, fs );

		// Pass it back through recognizer:
		String actual = lex.label( f2 ).toString();
		scLabel = sc.getLabel( lang ).toString();
		output.append("Scenario " + sc.getName() +
			      ": command=" + scLabel + 
			      ", output=" + actual);
		numTested++;
		if ( actual.equals(scLabel) ) {
		    numCorrect++;
		} else {
		    output.append("    *ERROR*");
		}

		output.append("\n");
		// Print out full word probability vector only if
		// detail is true.
		if ( wantDetail ) {
		    output.append( "Linking f-struct (pre-execution):\n\t" + 
				   f1.toString() );
		    output.append( "Linking f-struct (post-execution):\n\t" + 
				   f2.toString() );
		    Vector v = lex.allSensePosteriors( f2 );
		    for (int j=0; j<v.size(); j++) {
			output.append("  ");
			output.append( ((CandidateSense)v.
					elementAt(j)).toString() );
		    } // end for over CandidateSense vector from
		      // allSensePosteriors.
		}
	    } else {
		output.append("WARNING: scenario " + sc.getName() +
			      " is unlabelled in language " + lang +
			      ".\n");
	    }
	} // end for over scenarios to recognize.

	output.append("Correctly labelled " + numCorrect + " of " +
		      numTested + " test scenarios");
	if (numTested > 0) {
	    output.append(" (" +
			  ((int)((double)numCorrect / (double)numTested * 100)) + 
			  "%).\n");
	}
	output.append("Obey test done.\n\n");
    }

    private Fstruct execute(Fstruct f1, Fstruct w)
    {
	Fstruct f2 = (Fstruct)f1.clone();
	// Fill in needed defaults in f2:
	// TBD...
	return f2;
    }


}



